# Generated by roxygen2: do not edit by hand

export(TimeTreeFecher)
export(runshinyWGD)
importFrom(ape,as.phylo)
importFrom(ape,write.nexus)
importFrom(ape,write.tree)
importFrom(data.table,copy)
importFrom(dplyr,"%>%")
importFrom(dplyr,arrange)
importFrom(dplyr,bind_rows)
importFrom(dplyr,distinct)
importFrom(dplyr,filter)
importFrom(dplyr,group_by)
importFrom(dplyr,lag)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,row_number)
importFrom(dplyr,rowwise)
importFrom(dplyr,select)
importFrom(dplyr,summarise)
importFrom(fs,file_temp)
importFrom(graphics,hist)
importFrom(htmltools,tags)
importFrom(httr,GET)
importFrom(httr,content)
importFrom(jsonlite,fromJSON)
importFrom(seqinr,getLength)
importFrom(seqinr,read.fasta)
importFrom(seqinr,write.fasta)
importFrom(shiny,incProgress)
importFrom(shinyalert,shinyalert)
importFrom(stats,aggregate)
importFrom(stats,as.dist)
importFrom(stats,complete.cases)
importFrom(stats,cor)
importFrom(stats,cutree)
importFrom(stats,fft)
importFrom(stats,hclust)
importFrom(stats,kmeans)
importFrom(stats,median)
importFrom(stats,na.omit)
importFrom(stats,p.adjust)
importFrom(stats,pchisq)
importFrom(stats,ppois)
importFrom(stats,predict)
importFrom(stats,quantile)
importFrom(stats,smooth.spline)
importFrom(stats,var)
importFrom(stringr,regex)
importFrom(stringr,str_detect)
importFrom(stringr,str_trim)
importFrom(tidyr,fill)
importFrom(utils,read.delim)
importFrom(utils,read.table)
importFrom(utils,write.table)
importFrom(vroom,vroom)
